@@ -1,5 +1,8 @@
 
+import copy
+import asyncio
 from pathlib import PureWindowsPath
+
 from aiosmb.commons.interfaces.file import SMBFile
 from aiosmb.wintypes.access_mask import *
 from aiosmb.wintypes.fscc.FileAttributes import FileAttributes
@@ -22,7 +25,7 @@ class SMBDirectory:
 		self.allocation_size = None
 		self.attributes = None
 		self.file_id = None
-		self.sid = None
+		self.security_descriptor = None
 		
 		self.files = {}
 		self.subdirs = {}
@@ -124,7 +127,7 @@ class SMBDirectory:
 			return False, e
 
 	async def get_security_descriptor(self, connection):
-		if self.sid is None:
+		if self.security_descriptor is None:
 			file_id = None
 			try:
 				tree_id = self.tree_id
@@ -143,7 +146,7 @@ class SMBDirectory:
 				if err is not None:
 					raise err
 				
-				self.sid, err = await connection.query_info(
+				self.security_descriptor, err = await connection.query_info(
 					tree_id, 
 					file_id,
 					info_type = QueryInfoType.SECURITY, 
@@ -163,7 +166,7 @@ class SMBDirectory:
 					await connection.tree_disconnect(tree_id)
 
 
-		return self.sid, None
+		return self.security_descriptor, None
 	
 
 	def get_console_output(self):
@@ -217,27 +220,131 @@ class SMBDirectory:
 	async def delete_subdir(self, dir_name):
 		raise Exception('delete subdir not implemented!')
 
-	async def list_r(self, connection, depth = 3):
+	async def list_r(self, connection, depth = 3, maxentries = None, fetch_dir_sd = False, fetch_file_sd = False, exclude_dir = []):
 		"""
+		recursive list files and folders
 		Beware this will clear out the lists of files/folders to save memory!
 		"""
 		if depth == 0:
 			return
 		depth -= 1
-		_, err = await self.list(connection)
-		if err is not None:
-			yield self.unc_path, 'dir', err
+		ctr = 0
 
-		for file_entry in self.files:
-			yield self.files[file_entry].unc_path, 'file', None
-		
-		for directory in self.subdirs:
-			yield self.subdirs[directory].unc_path, 'dir', None
-			async for e,t,err in self.subdirs[directory].list_r(connection, depth):
-				yield e,t,err
-		
-		self.subdirs = {}
+		async for obj, otype, err in self.list_gen(connection):
+			await asyncio.sleep(0)
+			if otype == 'dir' and fetch_dir_sd is True and obj.name not in exclude_dir:
+				obj.tree_id = self.tree_id
+				_, err = await obj.get_security_descriptor(connection)
+				#if err is not None:
+				#	print(err)
+			
+			if otype == 'file' and fetch_file_sd is True:
+				obj.tree_id = self.tree_id
+				_, err = await obj.get_security_descriptor(connection)
+				#if err is not None:
+				#	print(err)
+
+			yield obj, otype, err
+			
+			ctr += 1
+			if err is not None:
+				break
+			
+			if ctr == maxentries:
+				yield self, 'maxed', None
+				break
+			
+			if otype == 'dir' and obj.name not in exclude_dir:
+				obj.tree_id = self.tree_id
+				async for e,t,err in obj.list_r(connection, depth, maxentries = maxentries, exclude_dir = exclude_dir):
+					yield e,t,err
+					await asyncio.sleep(0)
+
+
+	async def list_gen(self, connection):
+		"""
+		Lists all files and folders in the directory, yields the results as they arrive
+		directory: SMBDirectory
+		DOESN'T fill the SMBDirectory's data
+		"""
 		self.files = {}
+		self.subdirs = {}
+		
+		desired_access = FileAccessMask.FILE_READ_DATA
+		share_mode = ShareAccess.FILE_SHARE_READ
+		create_options = CreateOptions.FILE_DIRECTORY_FILE | CreateOptions.FILE_SYNCHRONOUS_IO_NONALERT
+		file_attrs = 0
+		create_disposition = CreateDisposition.FILE_OPEN
+
+	
+		if not self.tree_id:
+			tree_entry, err = await connection.tree_connect(self.get_share_path())
+			if err is not None:
+				yield self, 'dir', err
+				return
+			self.tree_id = tree_entry.tree_id
+		
+		file_id, err = await connection.create(self.tree_id, self.fullpath, desired_access, share_mode, create_options, create_disposition, file_attrs)
+		if err is not None:
+			yield self, 'dir', err
+			return
+		try:
+			while True:
+				await asyncio.sleep(0)
+				fileinfos, err = await connection.query_directory(self.tree_id, file_id)
+				if err is not None:
+					raise err
+				if not fileinfos:
+					break
+				for info in fileinfos:
+					if info.FileAttributes & FileAttributes.FILE_ATTRIBUTE_DIRECTORY:
+						dirname = info.FileName 
+						if info.FileName in ['.','..']:
+							continue
+						subdir = SMBDirectory()
+						subdir.tree_id = self.tree_id
+						if self.fullpath != '':
+							subdir.fullpath = '%s\\%s' % (self.fullpath, info.FileName)	
+						else:
+							subdir.fullpath = info.FileName
+						subdir.unc_path = '%s\\%s' % (self.unc_path, info.FileName)
+						subdir.parent_dir = self
+						subdir.name = info.FileName
+						subdir.creation_time = info.CreationTime
+						subdir.last_access_time = info.LastAccessTime
+						subdir.last_write_time = info.LastWriteTime
+						subdir.change_time = info.ChangeTime
+						subdir.allocation_size = info.AllocationSize
+						subdir.attributes = info.FileAttributes
+						
+						yield subdir, 'dir', None
+						
+					else:
+						file = SMBFile()
+						file.tree_id = self.tree_id
+						file.parent_dir = None
+						if self.fullpath != '':
+							file.fullpath = '%s\\%s' % (self.fullpath, info.FileName)
+							file.unc_path = '%s\\%s' % (self.unc_path, info.FileName)
+						else:
+							file.fullpath = info.FileName
+							file.unc_path = '%s\\%s' % (self.unc_path, info.FileName)
+						file.name = info.FileName
+						file.size = info.EndOfFile
+						file.creation_time = info.CreationTime
+						file.last_access_time = info.LastAccessTime
+						file.last_write_time = info.LastWriteTime
+						file.change_time = info.ChangeTime
+						file.allocation_size = info.AllocationSize
+						file.attributes = info.FileAttributes
+						yield file, 'file', None
+			
+		except Exception as e:
+			yield self, 'dir', e
+			return
+		finally:
+			if file_id is not None:
+				await connection.close(self.tree_id, file_id)
 
 	async def list(self, connection):
 		"""
