@@ -2,13 +2,21 @@
 import enum
 
 from aiosmb.dcerpc.v5.common.connection.smbdcefactory import SMBDCEFactory
+from aiosmb.connection import SMBConnection
+from aiosmb.dcerpc.v5.connection import DCERPC5Connection
 from aiosmb.dcerpc.v5 import rrp
-from aiosmb.dcerpc.v5.interfaces.servicemanager import *
 from aiosmb import logger
 from aiosmb.dcerpc.v5 import system_errors
 from aiosmb.wintypes.dtyp.structures.filetime import FILETIME
 from winacl.dtyp.security_descriptor import SECURITY_DESCRIPTOR
 from aiosmb.wintypes.dtyp.constrcuted_security.security_information import SECURITY_INFORMATION
+from aiosmb.dcerpc.v5.rpcrt import RPC_C_AUTHN_LEVEL_NONE,\
+	RPC_C_AUTHN_LEVEL_CONNECT,\
+	RPC_C_AUTHN_LEVEL_CALL,\
+	RPC_C_AUTHN_LEVEL_PKT,\
+	RPC_C_AUTHN_LEVEL_PKT_INTEGRITY,\
+	RPC_C_AUTHN_LEVEL_PKT_PRIVACY,\
+	DCERPCException, RPC_C_AUTHN_GSS_NEGOTIATE
 
 class HKEY(enum.Enum):
 	CLASSES_ROOT = 0 #Registry entries subordinate to this key define types (or classes) of documents and the properties associated with those types. Shell and COM applications use the information stored under this key.
@@ -72,9 +80,10 @@ def reg_decode_value(value_type, value_data):
 	
 	return value_data
 
-class RRP:
-	def __init__(self, connection):
-		self.connection = connection		
+class RRPRPC:
+	def __init__(self):
+		self.service_pipename = r'\winreg'
+		self.service_uuid = rrp.MSRPC_UUID_RRP
 		self.dce = None
 		self.handles = {}
 		self.__current_handle_id = 0
@@ -99,20 +108,6 @@ class RRP:
 	async def __aexit__(self, exc_type, exc, traceback):
 		await self.close()
 		return True, None
-	
-	async def connect(self):
-		try:
-			rpctransport = SMBDCEFactory(self.connection,  filename=r'\winreg')
-			self.dce = rpctransport.get_dce_rpc()
-			_, err = await self.dce.connect()
-			if err is not None:
-				raise err
-			_, err = await self.dce.bind(rrp.MSRPC_UUID_RRP)
-			if err is not None:
-				raise err
-			return True, None
-		except Exception as e:
-			return None, e
 
 	async def close(self):
 		try:
@@ -123,6 +118,48 @@ class RRP:
 		except Exception as e:
 			return False, e
 
+	@staticmethod
+	async def from_rpcconnection(connection:DCERPC5Connection, auth_level = None, open:bool = True, perform_dummy:bool = False):
+		try:
+			service = RRPRPC()
+			service.dce = connection
+			
+			service.dce.set_auth_level(auth_level)
+			if auth_level is None:
+				service.dce.set_auth_level(RPC_C_AUTHN_LEVEL_PKT_PRIVACY) #secure default :P 
+			
+			_, err = await service.dce.connect()
+			if err is not None:
+				raise err
+			
+			_, err = await service.dce.bind(service.service_uuid)
+			if err is not None:
+				raise err
+				
+			return service, None
+		except Exception as e:
+			return False, e
+	
+	@staticmethod
+	async def from_smbconnection(connection:SMBConnection, auth_level = None, open:bool = True, perform_dummy:bool = False):
+		"""
+		Creates the connection to the service using an established SMBConnection.
+		This connection will use the given SMBConnection as transport layer.
+		"""
+		try:
+			if auth_level is None:
+				#for SMB connection no extra auth needed
+				auth_level = RPC_C_AUTHN_LEVEL_NONE
+			rpctransport = SMBDCEFactory(connection, filename=RRPRPC().service_pipename)		
+			service, err = await RRPRPC.from_rpcconnection(rpctransport.get_dce_rpc(), auth_level=auth_level, open=open, perform_dummy = perform_dummy)	
+			if err is not None:
+				raise err
+
+			return service, None
+		except Exception as e:
+			return None, e
+
+
 	async def CloseKey(self, key):
 		try:
 			key = await self.__get_rawhandle(key)
@@ -385,7 +422,27 @@ class RRP:
 				return None, None, OSError(e.get_error_code(), system_errors.ERROR_MESSAGES[e.get_error_code()][1])
 			return None, e
 
+	######### custom methods
 
+	async def ListUsers(self, users_handle = None):
+		try:
+			if users_handle is None:
+				users_handle, err = await self.ConnectRegistry(HKEY.USERS)
+				if err is not None:
+					raise err
+			
+			users = []
+			for i in range(255):
+				res, err = await self.EnumKey(users_handle, i)
+				if err is not None:
+					if err.errno == 259: #no more data is available
+						break
+					raise err
+				if res.startswith('S-1-5-') is True:
+					users.append(res[:-1])
+			return users, None
+		except Exception as e:
+			return None, e
 
 
 ################## CUT HERE
@@ -474,12 +531,12 @@ async def ConnectRegistryTest(rs):
 		print(value_data)
 
 async def amain():
-	from aiosmb.commons.connection.url import SMBConnectionURL
+	from aiosmb.commons.connection.factory import SMBConnectionFactory
 	from aiosmb.connection import SMBConnection
 	import traceback
 
-	url = 'smb2+kerberos-password://TEST\\Administrator:QLFbT8zkiFGlJuf0B3Qq@win2019ad.test.corp?serverip=10.10.10.2&dc=10.10.10.2'
-	su = SMBConnectionURL(url)
+	url = 'smb2+ntlm-password://TEST\\Administrator:Passw0rd!1@10.10.10.2'
+	su = SMBConnectionFactory.from_url(url)
 	conn = su.get_connection()
 
 	_, err = await conn.login()
@@ -495,7 +552,23 @@ async def amain():
 		return
 	print('RRP Connected!')
 
+	users_handle, err = await rs.ConnectRegistry(HKEY.USERS)
+	if err is not None:
+		print('users error! %s' % err)
+		return
 
+	print('users_handle %s' % users_handle)
+	for i in range(10):
+		res, err = await rs.EnumKey(users_handle, i)
+		if err is not None:
+			if err.errno == 259: #no more data is available
+				break
+			print(err)
+			print(traceback.format_tb(err.__traceback__))
+		print(res)
+
+	"""
+	
 	await ConnectRegistryTest(rs)
 
 
@@ -543,6 +616,7 @@ async def amain():
 	if err is not None:
 		print(err)
 	print(res)
+	"""
 
 if __name__ == '__main__':
 	asyncio.run(amain())
\ No newline at end of file
