@@ -1,4 +1,4 @@
-from aiosmb.protocol.smb.commands.negotiate_req import SMB_COM_NEGOTIATE_REQ
+from aiosmb.protocol.smb.commands.negotiate import SMB_COM_NEGOTIATE_REQ, SMB_COM_NEGOTIATE_REPLY
 
 #add new commands as they are implemented.
 #do not forget to add the same commands to command_cdoes as well!
@@ -6,5 +6,6 @@ from aiosmb.protocol.smb.commands.negotiate_req import SMB_COM_NEGOTIATE_REQ
 
 __all__ = [
 	'SMB_COM_NEGOTIATE_REQ',
+	'SMB_COM_NEGOTIATE_REPLY',
 
 ]
\ No newline at end of file
